unit restresource;

interface
uses
  System.SysUtils,
  System.Classes,
  System.JSON,
  EMS.ResourceAPI,
  EMS.ResourceTypes,
  FireDAC.Comp.Client;

type
  TRESTResource = class(TComponent)
  private
    fFDConnection: TFDConnection;
    fTableName: string;
    fKeyField: string;
  private
    function QueryToJSONArray(qry: TFDQuery): TJSONArray;
  public
    procedure Get(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
    procedure GetItem(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
    procedure Post(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
    procedure PutItem(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
    procedure DeleteItem(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
  public
    constructor Create( aOwner: TComponent ); reintroduce;
    destructor Destroy; override;
  published
    property FDConnection: TFDConnection read fFDConnection write fFDConnection;
    property TableName: string read fTableName write fTableName;
    property KeyField: string read fKeyField write fKeyField;
  end;

procedure Register;

implementation
uses
  data.db;

procedure Register;
begin
  RegisterComponents('Samples', [TRESTResource]);
end;

{ TRESTResource }

constructor TRESTResource.Create(aOwner: TComponent);
begin
  inherited Create( aOwner );
  fFDConnection := nil;
  fTableName := '';
  fKeyField := '';
end;

procedure TRESTResource.DeleteItem(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  LItem: string;
begin
  LItem := ARequest.Params.Values['item'];
end;

destructor TRESTResource.Destroy;
begin
  fFDConnection := nil;
  inherited Destroy;
end;

function TRESTResource.QueryToJSONArray( qry: TFDQuery ): TJSONArray;
var
  aRecord: TJSONObject;
  idx: Integer;
begin
  Result := TJSONArray.Create;
  if qry.RecordCount=0 then begin
    exit;
  end;
  qry.First;
  while not (qry.Eof) do begin
    aRecord := TJSONObject.Create;
    if qry.Fields.Count>0 then begin
      for idx := 0 to pred(qry.Fields.Count) do begin
        //- Some guesswork is done regarding the field types...
        case qry.Fields[idx].DataType of
          //- String types.
          TFieldType.ftFixedChar,
          TFieldType.ftWideString,
          TFieldType.ftString,
          TFieldType.ftMemo,
          TFieldType.ftFmtMemo: begin
            aRecord.AddPair(qry.Fields[idx].fieldName,TJSONString.Create(qry.Fields[idx].AsString));
          end;
          //- Date/Time types (translate as string)
          TFieldType.ftDate,
          TFieldType.ftTime,
          TFieldType.ftDateTime,
          TFieldType.ftTimeStamp,
          TFieldType.ftFixedWideChar,
          TFieldType.ftWideMemo,
          TFieldType.ftOraTimeStamp,
          TFieldType.ftOraInterval,
          TFieldType.ftTimeStampOffset: begin
            aRecord.AddPair(qry.Fields[idx].fieldName,TJSONString.Create(qry.Fields[idx].AsString));
          end;
          //- Integer types
          TFieldType.ftAutoInc,
          TFieldType.ftLargeint,
          TFieldType.ftSmallint,
          TFieldType.ftInteger,
          TFieldType.ftWord,
          TFieldType.ftLongWord,
          TFieldType.ftShortint,
          TFieldType.ftByte: begin
            aRecord.AddPair(qry.Fields[idx].fieldName,TJSONNumber.Create(qry.Fields[idx].AsInteger));
          end;
          //- Floating point types.
          TFieldType.ftFloat,
          TFieldType.ftCurrency,
          TFieldType.ftBCD,
          TFieldType.ftFMTBcd,
          TFieldType.ftExtended,
          TFieldType.ftSingle: begin
            aRecord.AddPair(qry.Fields[idx].fieldName,TJSONNumber.Create(qry.Fields[idx].AsExtended));
          end;
          //- Boolean type
          TFieldType.ftBoolean: begin
            if qry.Fields[idx].AsBoolean then begin
              aRecord.AddPair(qry.Fields[idx].fieldName,TJSONBool.Create(true));
            end else begin
              aRecord.AddPair(qry.Fields[idx].fieldName,TJSONBool.Create(false));
            end;
          end;
          else begin
            raise Exception.Create('Unsupported database field type.');
          end;
        end;
      end;
      Result.Add(aRecord);
      qry.Next;
    end;
  end;
end;


procedure TRESTResource.Get(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  qry: TFDQuery;
  Results: TJSONArray;
begin
  qry := TFDQuery.Create(nil);
  try
    qry.Connection := FDConnection;
    qry.SQL.Text := 'SELECT * FROM '+TableName;
    //- Activate query with error trapping
    try
      qry.Active := True;
    except
      on E: Exception do begin
        AResponse.StatusCode := 500; //- Internal Server Error.
        AResponse.Body.SetValue(TJSONString.Create(E.Message),TRUE);
        exit;
      end;
    end;
    if not qry.Active then begin
      AResponse.StatusCode := 500; //- Internal Server Error.
      AResponse.Body.SetValue(TJSONString.Create('Unknown internal error'),TRUE);
      exit;
    end;
    //-
    AResponse.StatusCode := 200;
    Results := QueryToJSONArray( qry );
    Aresponse.Body.SetValue(Results,True);
  finally
    qry.DisposeOf;
  end;
end;

procedure TRESTResource.GetItem(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  LItem: string;
  qry: TFDQuery;
  Results: TJSONArray;
begin
  LItem := ARequest.Params.Values['item'];
  qry := TFDQuery.Create(nil);
  try
    qry.Connection := FDConnection;
    qry.SQL.Text := 'SELECT * FROM '+TableName+' WHERE '+KeyField+'=:KeyField';
    qry.Params.ParamByName('KeyField').AsString := LItem;
    //- Activate query with error trapping
    try
      qry.Active := True;
    except
      on E: Exception do begin
        AResponse.StatusCode := 500; //- Internal Server Error.
        AResponse.Body.SetValue(TJSONString.Create(E.Message),TRUE);
        exit;
      end;
    end;
    if not qry.Active then begin
      AResponse.StatusCode := 500; //- Internal Server Error.
      AResponse.Body.SetValue(TJSONString.Create('Unknown internal error'),TRUE);
      exit;
    end;
    //-
    AResponse.StatusCode := 200;
    Results := QueryToJSONArray( qry );
    Aresponse.Body.SetValue(Results,True);
  finally
    qry.DisposeOf;
  end;
end;


procedure TRESTResource.Post(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
begin

end;

procedure TRESTResource.PutItem(const AContext: TEndpointContext; const ARequest: TEndpointRequest; const AResponse: TEndpointResponse);
var
  LItem: string;
begin
  LItem := ARequest.Params.Values['item'];
end;

end.
